@@ -3,6 +3,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <getopt.h>
+#include <locale.h>
 #include <sys/file.h>
 #include <sys/ioctl.h>
 
@@ -111,7 +112,7 @@ int show_set(AVLTree *set, char *fmt, ...) {
 static void showinfo(symbol_t pkg) {
 	AVLTree t;
 	AVLNode n;
-	AVLInitTree(&t, (AVLCompare)strcmp, NULL);
+	AVLInitTree(&t, (AVLCompare)strcmp);
 	AVLInitNode(&n, pkg);
 	AVLInsertTopNode(&t, &n);
 	xargs(&t, InfoCmd);
@@ -282,8 +283,8 @@ void add_provider(struct package *pkg, struct package *ppkg) {
 
 	if(ppkg->provider_count) {
 		/* Remove multiple alternative dependencies to a dependency loop */
-		AVLInitTree(&depends, (AVLCompare)symcmp, NULL);
-		AVLInitTree(&dependents, (AVLCompare)symcmp, NULL);
+		AVLInitTree(&depends, (AVLCompare)symcmp);
+		AVLInitTree(&dependents, (AVLCompare)symcmp);
 		all_depends(pkg->name, &depends, ENoAlternatives, EAllDepends);
 		all_dependents(pkg->name, &dependents, ENoAlternatives);
 		for(c = ppkg->providers.head; c; c = p) {
@@ -299,8 +300,6 @@ void add_provider(struct package *pkg, struct package *ppkg) {
 				ppkg->provider_count--;
 			}
 		}
-		AVLFreeNodes(&depends);
-		AVLFreeNodes(&dependents);
 	}
 
 	if (pkg) {
@@ -373,9 +372,8 @@ void init_packages(int ignore_defaults) {
  */
 void count_orphan_depends(struct package *pkg) {
 	AVLTree visited;
-	AVLInitTree(&visited, (AVLCompare)symcmp, NULL);
+	AVLInitTree(&visited, (AVLCompare)symcmp);
 	pkg->orphan_depends = all_depends(pkg->name, &visited, ECommittedAlternatives, EOrphansOnly);
-	AVLFreeNodes(&visited);
 }
 
 /* --show-providers option */
@@ -406,7 +404,7 @@ void show_depends(symbol_t name) {
 		return;
 	}
 
-	AVLInitTree(&visited, (AVLCompare)strcasecmp, NULL);
+	AVLInitTree(&visited, (AVLCompare)strcasecmp);
 	all_depends(name, &visited, ECommittedAlternatives, EAllDepends);
 
 	for(c = visited.head; c; c = p) {
@@ -416,8 +414,6 @@ void show_depends(symbol_t name) {
 	}
 	if(!show_set(&visited, _("Package %s depends on:"), name))
 		printf(_("Package %s has no depends.\n"), name);
-
-	AVLFreeNodes(&visited);
 }
 
 /* --show-dependents option */
@@ -432,7 +428,7 @@ void show_dependents(symbol_t name) {
 		return;
 	}
 
-	AVLInitTree(&dependents, (AVLCompare)symcmp, NULL);
+	AVLInitTree(&dependents, (AVLCompare)symcmp);
 	all_dependents(name, &dependents, ECommittedAlternatives);
 
 	for(c = dependents.head; c; c = p) {
@@ -446,8 +442,6 @@ void show_dependents(symbol_t name) {
 	if (!show_set(&dependents, _("The following %d packages on keeper list rely on %s:"), count, name))
 		printf(_("Packages on keeper list do not rely on %s.\n"), name);
 	printf(_("Packages kept by default rules %s %s.\n"), others ? _("rely on") : _("do not rely on"), name);
-
-	AVLFreeNodes(&dependents);
 }
 
 /* Find a set of orphan packages that are dependencies of orphan
@@ -462,8 +456,8 @@ int related_set(symbol_t name, AVLTree *set) {
 	struct package *pkg, *ppkg;
 	int count = 0;
 
-	AVLInitTree(&ancestors, (AVLCompare)symcmp, NULL);
-	AVLInitTree(&eliminate, (AVLCompare)symcmp, NULL);
+	AVLInitTree(&ancestors, (AVLCompare)symcmp);
+	AVLInitTree(&eliminate, (AVLCompare)symcmp);
 
 	all_depends(name, set, ECommittedAlternatives, EOrphansOnly);
 	AVLInsert(&ancestors, name);
@@ -491,8 +485,6 @@ int related_set(symbol_t name, AVLTree *set) {
 			count++;
 	}
 
-	AVLFreeNodes(&ancestors);
-	AVLFreeNodes(&eliminate);
 	return count;
 }
 
@@ -507,12 +499,10 @@ void show_related(symbol_t name) {
 		return;
 	}
 
-	AVLInitTree(&related, (AVLCompare)strcasecmp, NULL);
+	AVLInitTree(&related, (AVLCompare)strcasecmp);
 	count = related_set(name, &related);
 	if(!show_set(&related, _("The following %d packages are brought in by %s:"), count, name))
 		printf(_("No packages are brought in by %s.\n"), name);
-
-	AVLFreeNodes(&related);
 }
 
 /* Push a symbol onto undo stack */
@@ -529,14 +519,13 @@ symbol_t pop_undo(void) {
 	if(undo) {
 		symbol_t name = undo->name;
 		undo_stack = undo->prev;
-		free(undo);
 		return name;
 	}
 	return 0;
 }
 
 int main(int argc, char **argv) {
-	AVLTree trees[9], *installs, *removes, *orphans, *related;
+	AVLTree *installs, *removes, *orphans, *related;
 	AVLNode *c, *p;
 	struct package *pkg, *ppkg;
 	int i, option_index, ignore_defaults = 0, show_option = 0, nokeeperfile = 0, upgrade = 0, mark_only = getuid();
@@ -548,17 +537,17 @@ int main(int argc, char **argv) {
 	textdomain (PACKAGE);
 
 	/* global trees */
-	packages  = AVLInitTree(trees,   (AVLCompare)pkgcmp, (AVLFreeItem)free_package);
-	keepers   = AVLInitTree(trees+1, (AVLCompare)strcasecmp, NULL);
-	nokeepers = AVLInitTree(trees+2, (AVLCompare)strcasecmp, NULL);
+	packages  = AVLAllocTree((AVLCompare)pkgcmp);
+	keepers   = AVLAllocTree((AVLCompare)strcasecmp);
+	nokeepers = AVLAllocTree((AVLCompare)strcasecmp);
 
 	/* local trees */
-	postpone  = AVLInitTree(trees+3, (AVLCompare)strcasecmp, NULL);
-	keeplist  = AVLInitTree(trees+4, (AVLCompare)symcmp, NULL);
-	installs  = AVLInitTree(trees+5, (AVLCompare)strcasecmp, NULL);
-	removes   = AVLInitTree(trees+6, (AVLCompare)strcasecmp, NULL);
-	orphans   = AVLInitTree(trees+7, (AVLCompare)pkgsortcmp, NULL);
-	related   = AVLInitTree(trees+8, (AVLCompare)strcasecmp, NULL);
+	postpone  = AVLAllocTree((AVLCompare)strcasecmp);
+	keeplist  = AVLAllocTree((AVLCompare)symcmp);
+	installs  = AVLAllocTree((AVLCompare)strcasecmp);
+	removes   = AVLAllocTree((AVLCompare)strcasecmp);
+	orphans   = AVLAllocTree((AVLCompare)pkgsortcmp);
+	related   = AVLAllocTree((AVLCompare)strcasecmp);
 
 	readconfig(NULL);
 
@@ -738,7 +727,7 @@ int main(int argc, char **argv) {
 		/* Add all dependencies to install list (--upgrade) */
 		if(upgrade) {
 			AVLTree install_depends;
-			AVLInitTree(&install_depends, (AVLCompare)symcmp, NULL);
+			AVLInitTree(&install_depends, (AVLCompare)symcmp);
 			for(c = installs->head; c; c = c->next)
 				if(pkg_find(c->item))
 					all_depends(c->item, &install_depends, ECommittedAlternatives, EAllDepends);
@@ -815,7 +804,6 @@ int main(int argc, char **argv) {
 					(ppkg = pkg_find(base)) && ppkg->installed) {
 				if(AVLSearch(keeplist, ppkg->name)) {
 					add_keeplist(pkg);
-					free(c); /* The node is unlinked so we can just delete it */
 					continue;
 				}
 				if(AVLSearch(removes, ppkg->name)) {
